use std::collections::HashSet;

use serde::{Deserialize, Serialize};

pub use self::{enums::{Flag, InstallAction}, meta::*};

pub mod api;
pub mod meta;
pub mod enums;

#[derive(Debug, Serialize, Deserialize)]
pub struct Manifest {
    pub addonscript: AddonScript,
    pub id: String,
    pub namespace: String,
    pub version: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub files : Option<Vec<File>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub relations: Option<Vec<Relation>>,
    pub flags: Flags,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub repositories: Option<Vec<Repository>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub meta: Option<Meta>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct VersionlessManifest {
    pub addonscript: AddonScript,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct AddonScript {
    pub version: u8,
}

impl Default for AddonScript {
    fn default() -> Self {
        Self { version: 2 }
    }
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Repository {
    pub namespace: String,
    pub instances: Vec<String>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Relation {
    pub id: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub namespace: Option<String>,
    pub version: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub repositories: Option<Vec<String>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub flags: Option<Flags>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct File {  
    pub qualifier: String,
    pub link: Vec<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub install: Option<Vec<Install>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub flags: Option<Flags>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub hashes: Option<Hashes>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Hashes {
    #[serde(skip_serializing_if = "Option::is_none")]
    pub sha1: Option<String>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Install {
    pub action: InstallAction,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub args: Option<Vec<String>>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Flags {
    #[serde(skip_serializing_if = "Option::is_none")]
    pub client: Option<HashSet<Flag>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub server: Option<HashSet<Flag>>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub both: Option<HashSet<Flag>>,
}

impl VersionlessManifest {
    pub fn is_version_valid(&self) -> bool {
        self.addonscript.version == 2
    }
}

impl Flags {
    
    pub fn has_client_flag(&self, flag: &Flag) -> bool {
        if let Some(flags) = &self.client {
            if flags.contains(flag) {
                return true;
            }
        }
        if let Some(flags) = &self.both {
            if flags.contains(flag) {
                return true;
            }
        }
        return false;
    }

    pub fn has_server_flag(&self, flag: &Flag) -> bool {
        if let Some(flags) = &self.server {
            if flags.contains(flag) {
                return true;
            }
        }
        if let Some(flags) = &self.both {
            if flags.contains(flag) {
                return true;
            }
        }
        return false;
    }

}